@@ -27,7 +27,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
-#include "ctkimage.h"
+#include "clocks_banner.h"
 
 #include "ctkclocks.h"
 
@@ -279,11 +279,11 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
     GtkWidget *menu_item;
     GtkWidget *scrollWin;
     GtkWidget *event;
+    GdkColor   color;
 
     GtkWidget *label;   
 
     GtkWidget *frame;
-    GtkWidget *banner;
     GtkWidget *hbox;
     GtkWidget *vbox;
 
@@ -296,8 +296,7 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
     NVCTRLAttributeValidValuesRec ranges_3D;
 
     Bool overclocking_enabled;
-    Bool auto_detection_available = FALSE;
-    Bool probing_optimal = FALSE;
+    Bool probing_optimal;
     Bool can_access_2d_clocks;
     Bool can_access_3d_clocks;
    
@@ -319,18 +318,13 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
     
     if ( overclocking_enabled ) {
         ret = NvCtrlGetAttribute(handle,
-                                 NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION,
+                                 NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_STATE,
                                  &value);
-        if ( ret == NvCtrlSuccess ) {
-            ret = NvCtrlGetAttribute(handle,
-                                     NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_STATE,
-                                     &value);
-            if ( ret != NvCtrlSuccess )
-                return NULL;
-            probing_optimal =
-                (value == NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_STATE_BUSY);
-            auto_detection_available = TRUE;
-        }
+        if ( ret != NvCtrlSuccess )
+            return NULL;
+        probing_optimal = (value==NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_STATE_BUSY);
+    } else {
+        probing_optimal = FALSE;
     }
 
     /* Can we access the 2D clocks? */
@@ -371,7 +365,6 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
     ctk_object->ctk_config           = ctk_config;
     ctk_object->license_accepted     = // = overclocking_enabled
     ctk_object->overclocking_enabled = overclocking_enabled;
-    ctk_object->auto_detection_available  = auto_detection_available;
     ctk_object->probing_optimal      = probing_optimal;
 
     /* Create the Clock menu widget */
@@ -517,8 +510,7 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
         gtk_widget_set_sensitive(ctk_object->detect_button, False);
     } else {
         gtk_widget_set_sensitive(ctk_object->detect_button,
-                                 overclocking_enabled &&
-                                 auto_detection_available && !probing_optimal);
+                                 overclocking_enabled && !probing_optimal);
     }
 
     /* Create the Reset hardware button widget */
@@ -561,8 +553,8 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
     event = gtk_event_box_new();
     ctk_object->license_window = scrollWin;
 
-    gtk_widget_modify_fg(event, GTK_STATE_NORMAL, &(event->style->text[GTK_STATE_NORMAL]));
-    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, &(event->style->base[GTK_STATE_NORMAL]));
+    gdk_color_parse("white", &color);
+    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, &color);
 
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
@@ -625,8 +617,29 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
 
     gtk_box_set_spacing(GTK_BOX(ctk_object), 10);
 
-    banner = ctk_banner_image_new(BANNER_ARTWORK_CLOCK);
-    gtk_box_pack_start(GTK_BOX(object), banner, FALSE, FALSE, 0);
+
+    { /* Banner image */
+        const nv_image_t *image_data   = &clocks_banner_image;
+        guint8           *image_buffer = decompress_image_data(image_data);
+        GdkPixbuf        *image_pixbuf = 
+            gdk_pixbuf_new_from_data(image_buffer, GDK_COLORSPACE_RGB,
+                                     FALSE, 8, image_data->width,
+                                     image_data->height,
+                                     image_data->width * image_data->bytes_per_pixel,
+                                     free_decompressed_image, NULL);
+        GtkWidget *image = gtk_image_new_from_pixbuf(image_pixbuf);
+
+        frame = gtk_frame_new(NULL);
+
+        hbox = gtk_hbox_new(FALSE, 0);
+
+        gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, 0);
+        
+        gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+        gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
+        
+        gtk_container_add(GTK_CONTAINER(frame), image);
+    }
 
     /* Add Overclocking checkbox */
 
@@ -695,6 +708,11 @@ GtkWidget* ctk_clocks_new(NvCtrlAttributeHandle *handle,
                      G_CALLBACK(clocks_received),
                      (gpointer) ctk_object);
     
+    g_signal_connect(G_OBJECT(ctk_event),
+                     CTK_EVENT_NAME(NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS),
+                     G_CALLBACK(clocks_received),
+                     (gpointer) ctk_object);
+    
     g_signal_connect(G_OBJECT(ctk_event),
                      CTK_EVENT_NAME(NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS),
                      G_CALLBACK(clocks_received),
@@ -835,12 +853,9 @@ static void sync_gui_sensitivity(CtkClocks *ctk_object)
                                __cancel_button_help);
 
     } else {
-        gboolean set_sensitive;
         gtk_button_set_label(GTK_BUTTON(ctk_object->detect_button), "Auto Detect");
-        set_sensitive = ((ctk_object->auto_detection_available) &&
-                         (ctk_object->clocks_being_modified == CLOCKS_3D))
-                        ? enabled : False;
-        gtk_widget_set_sensitive(ctk_object->detect_button, set_sensitive);
+        gtk_widget_set_sensitive(ctk_object->detect_button, 
+                                 (ctk_object->clocks_being_modified == CLOCKS_3D)?enabled:False);
         ctk_config_set_tooltip(ctk_object->ctk_config, ctk_object->detect_button,
                                __detect_button_help);
     }
@@ -1155,10 +1170,6 @@ static void sync_gui_to_modify_clocks(CtkClocks *ctk_object, int which_clocks)
                         GET_GPU_CLOCK(clk_ranges.u.range.min),
                         GET_GPU_CLOCK(clk_ranges.u.range.max));
 
-    g_signal_handlers_unblock_by_func(G_OBJECT(gtk_adjustment_gpu),
-                                      G_CALLBACK(adjustment_value_changed),
-                                      (gpointer) ctk_object);
-
     gtk_range = GTK_RANGE(CTK_SCALE(ctk_object->mem_clk_scale)->gtk_scale);
     gtk_adjustment_mem = GTK_ADJUSTMENT(gtk_range->adjustment);
 
@@ -1175,6 +1186,11 @@ static void sync_gui_to_modify_clocks(CtkClocks *ctk_object, int which_clocks)
                                       G_CALLBACK(adjustment_value_changed),
                                       (gpointer) ctk_object);
 
+    g_signal_handlers_unblock_by_func(G_OBJECT(gtk_adjustment_gpu),
+                                      G_CALLBACK(adjustment_value_changed),
+                                      (gpointer) ctk_object);
+
+
     /* Update the gui sensitivity */
     
     sync_gui_sensitivity(ctk_object);
