@@ -24,15 +24,14 @@
  
 #include <gtk/gtk.h>
 #include "ctkutils.h"
-                                                                                            
 #include <stdlib.h>
 
- 
-void add_table_row(GtkWidget *table,
-                   const gint row,
-                   const gint value_alignment,
-                   const gchar *name,
-                   const gchar *value)
+
+GtkWidget *add_table_row(GtkWidget *table,
+                         const gint row,
+                         const gint value_alignment,
+                         const gchar *name,
+                         const gchar *value)
 {
     GtkWidget *label;
 
@@ -48,5 +47,7 @@ void add_table_row(GtkWidget *table,
     gtk_misc_set_alignment(GTK_MISC(label), value_alignment, 0.5);
     gtk_table_attach(GTK_TABLE(table), label, 1, 2, row, row + 1,
                      GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+
+    return label;
 }
     
